﻿Imports GTA
Imports GTA.Math
Imports System
Imports System.IO
Imports System.Text
Imports System.Drawing
Imports System.Windows.Forms
Imports System.Threading.Tasks
Imports System.Collections.Generic

Public Class VehicleExportMissions
    Inherits Script
    Public RND As New Random

    Public MinigameStage As MinigameStages

    Public DropoffLocation As Vector3 = New Vector3(-234.4, -2661.3, 5.5)
    Public DropoffLocationBlip As Blip
    Public DoorLocation As Vector3 = New Vector3(-237.1, -2666.9, 8.9)
    Public DoorHeading As Integer = 93
    Public DoorBlip As Blip

    Public WantedCarIndex As Integer
    Public WantedCarEntity As Vehicle
    Public WantedCars(2) As Car
    Public RewardPercentages(2) As Single

    Public MaximumRewardPercentage As Single = 0.08

    Public isPlayerInWantedCar As Boolean = False
    Public isWantedCarMessageSeen As Boolean = False
    Public isVehicleDelivered As Boolean = False

    Public UI_Main_width As Integer = 320
    Public UI_Main_height As Integer = 250
    Public UI_Main_x As Integer = (1280 / 2) - (UI_Main_width / 2)
    Public UI_Main_y As Integer = (720 / 2) - (UI_Main_height / 2)
    Public UI_Main_ULCorner As New Point(UI_Main_x, UI_Main_y)

    Public UI_Main As New UIContainer(UI_Main_ULCorner, New Size(UI_Main_width, UI_Main_height), Color.FromArgb(170, 0, 0, 0)) With {.Enabled = False}

    Public UI_Debug As New UIContainer(New Point(0, 0), New Size(400, 300), Color.FromArgb(180, 0, 0, 0)) With {.Enabled = False}

    Public Enum MinigameStages
        LookingForCar
        DrivingToDropoffLocation
        WaitingToExitCar
        WalkingAway
    End Enum

    Public Sub New()
        MinigameStage = MinigameStages.LookingForCar
        GenerateNewWantedCarList()
        DoorBlip = World.CreateBlip(DoorLocation)
        GTA.Native.Function.Call(Native.Hash.SET_BLIP_AS_SHORT_RANGE, DoorBlip.Handle, True)
        'GTA.Native.Function.Call(Native.Hash.SET_BLIP_DISPLAY, DoorBlip, "Clever Eddie's Car Exchange")
        DoorBlip.Sprite = 50
        DoorBlip.Color = BlipColor.White
    End Sub

    Public Sub Update(ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Tick


        checkIfPlayerIsStandingAtWarehouse()

        checkIfPlayerIsDead()

        checkIfPlayerIsInWantedCar()
        checkIfPlayerIsAtDropoffLocation()
        checkIfPlayerHasLeftWantedCar()

        drawUI()

    End Sub

    Public Sub ResetVariables()
        WantedCarEntity = Nothing
        isPlayerInWantedCar = False
        isWantedCarMessageSeen = False
        isVehicleDelivered = False
        WantedCarIndex = -1
    End Sub

    Public Sub KeypressI(ByVal sender As Object, ByVal k As KeyEventArgs) Handles MyBase.KeyUp
        If k.KeyCode = Keys.I Then
        End If
    End Sub

    Public Sub FillWantedCarList()

        If WantedCars(0) Is Nothing Then
            Do
                Dim i As Integer = RND.Next(0, CarList.Count)
                WantedCars(0) = CarList(i)
                RewardPercentages(0) = RND.Next(10, MaximumRewardPercentage * 1000) / 1000
            Loop While WantedCars(0).DisplayName = WantedCars(1).DisplayName And WantedCars(0).DisplayName = WantedCars(2).DisplayName
        End If

        If WantedCars(1) Is Nothing Then
            Do
                Dim i As Integer = RND.Next(0, CarList.Count)
                WantedCars(1) = CarList(i)
                RewardPercentages(1) = RND.Next(10, MaximumRewardPercentage * 1000) / 1000
            Loop While WantedCars(1).DisplayName = WantedCars(0).DisplayName And WantedCars(1).DisplayName = WantedCars(2).DisplayName
        End If

        If WantedCars(2) Is Nothing Then
            Do
                Dim i As Integer = RND.Next(0, CarList.Count)
                WantedCars(2) = CarList(i)
                RewardPercentages(2) = RND.Next(10, MaximumRewardPercentage * 1000) / 1000
            Loop While WantedCars(2).DisplayName = WantedCars(0).DisplayName And WantedCars(2).DisplayName = WantedCars(1).DisplayName
        End If

    End Sub

    Public Sub GenerateNewWantedCarList()
        Dim i As Integer = RND.Next(0, CarList.Count)
        WantedCars(0) = CarList(i)
        RewardPercentages(0) = RND.Next(10, MaximumRewardPercentage * 1000) / 1000

        Do
            i = RND.Next(0, CarList.Count)
            WantedCars(1) = CarList(i)
            RewardPercentages(1) = RND.Next(10, MaximumRewardPercentage * 1000) / 1000
        Loop While WantedCars(1).DisplayName = WantedCars(0).DisplayName

        Do
            i = RND.Next(0, CarList.Count)
            WantedCars(2) = CarList(i)
            RewardPercentages(2) = RND.Next(10, MaximumRewardPercentage * 1000) / 1000
        Loop While WantedCars(2).DisplayName = WantedCars(0).DisplayName And WantedCars(2).DisplayName = WantedCars(1).DisplayName

    End Sub

    Public Sub drawUI()
        UI_Main.Items.Clear()

        UI_Main.Items.Add(New UIRectangle(New Point(0, 0), New Size(UI_Main_width, 45), Color.CadetBlue))
        UI_Main.Items.Add(New UIText("Clever Eddie's Car Exchange", New Point(10, 1), 0.8, Color.White, 1, False))
        UI_Main.Items.Add(New UIText("We're currently looking for the following vehicles:", New Point(10, 27), 0.25, Color.White, 0, False))
        UI_Main.Items.Add(New UIRectangle(New Point(0, 45), New Size(UI_Main_width, 1), Color.White))

        If WantedCars(0) IsNot Nothing Then
            UI_Main.Items.Add(New UIText(WantedCars(0).Name, New Point(10, 50), 0.6, Color.White, 0, False))
            UI_Main.Items.Add(New UIText("Reward: up to $" & (RewardPercentages(0) * WantedCars(0).Price).ToString("N0") & "  (" & Math.Round(RewardPercentages(0) * 100, 1) & "%)*", New Point(10, 80), 0.28, Color.White, 0, False))
        End If

        If WantedCars(1) IsNot Nothing Then
            UI_Main.Items.Add(New UIText(WantedCars(1).Name, New Point(10, 110), 0.6, Color.White, 0, False))
            UI_Main.Items.Add(New UIText("Reward: up to $" & (RewardPercentages(1) * WantedCars(1).Price).ToString("N0") & "  (" & Math.Round(RewardPercentages(1) * 100, 1) & "%)*", New Point(10, 140), 0.28, Color.White, 0, False))
        End If

        If WantedCars(2) IsNot Nothing Then
            UI_Main.Items.Add(New UIText(WantedCars(2).Name, New Point(10, 170), 0.6, Color.White, 0, False))
            UI_Main.Items.Add(New UIText("Reward: up to $" & (RewardPercentages(2) * WantedCars(2).Price).ToString("N0") & "  (" & Math.Round(RewardPercentages(2) * 100, 1) & "%)*", New Point(10, 200), 0.28, Color.White, 0, False))
        End If

        UI_Main.Items.Add(New UIText("*Any damage present at delivery will be deducted from your final payment.", New Point(10, 231), 0.2, Color.White, 0, False))

        UI_Main.Draw()


        UI_Debug.Items.Clear()
        UI_Debug.Items.Add(New UIText("Stage: " & MinigameStage.ToString, New Point(2, 18 * 0), 0.3, Color.White, 0, False))
        If Game.Player.Character.IsInVehicle = True Then
            UI_Debug.Items.Add(New UIText("Player Car: " & Game.Player.Character.CurrentVehicle.DisplayName, New Point(2, 18 * 1), 0.3, Color.White, 0, False))
            UI_Debug.Items.Add(New UIText("Stolen? " & Game.Player.Character.CurrentVehicle.IsStolen.ToString, New Point(2, 18 * 2), 0.3, Color.White, 0, False))
        Else
            UI_Debug.Items.Add(New UIText("Player Car: Not In Car", New Point(2, 18 * 1), 0.3, Color.White, 0, False))
            UI_Debug.Items.Add(New UIText("Stolen? Not In Car", New Point(2, 18 * 2), 0.3, Color.White, 0, False))
        End If
        UI_Debug.Items.Add(New UIText("Wanted Cars: " & WantedCars(0).DisplayName & "/" & WantedCars(1).DisplayName & "/" & WantedCars(2).DisplayName, New Point(2, 18 * 3), 0.3, Color.White, 0, False))
        UI_Debug.Items.Add(New UIText("Is player in wanted car: " & isPlayerInWantedCar.ToString, New Point(2, 18 * 4), 0.3, Color.White, 0, False))
        UI_Debug.Items.Add(New UIText("Distance to Warehouse: " & World.GetDistance(Game.Player.Character.Position, DropoffLocation), New Point(2, 18 * 5), 0.3, Color.White, 0, False))
        If WantedCarEntity IsNot Nothing Then
            UI_Debug.Items.Add(New UIText("Wanted car: " & WantedCarEntity.Handle & " / Delivered? " & isVehicleDelivered.ToString, New Point(2, 18 * 6), 0.3, Color.White, 0, False))
        End If

        UI_Debug.Draw()
    End Sub



    Public Sub checkIfPlayerIsStandingAtWarehouse()
        If Game.Player.Character.IsInVehicle = False Then
            If World.GetDistance(Game.Player.Character.Position, DoorLocation) < 1.5 Then
                If Game.Player.Character.Heading > DoorHeading - 15 And Game.Player.Character.Heading < DoorHeading + 15 Then
                    UI_Main.Enabled = True
                Else
                    UI_Main.Enabled = False
                End If
            Else
                UI_Main.Enabled = False
            End If
        End If
    End Sub

    Public Sub checkIfPlayerIsInWantedCar()
        If Game.Player.Character.IsInVehicle = True Then
            Dim PlayerCar As String = Game.Player.Character.CurrentVehicle.DisplayName

            If PlayerCar = WantedCars(0).DisplayName Or PlayerCar = WantedCars(1).DisplayName Or PlayerCar = WantedCars(2).DisplayName Then

                isPlayerInWantedCar = True

                If PlayerCar = WantedCars(0).DisplayName Then
                    WantedCarIndex = 0
                ElseIf PlayerCar = WantedCars(1).DisplayName Then
                    WantedCarIndex = 1
                ElseIf PlayerCar = WantedCars(2).DisplayName Then
                    WantedCarIndex = 2
                End If

                MinigameStage = MinigameStages.DrivingToDropoffLocation
                WantedCarEntity = Game.Player.Character.CurrentVehicle
                If isWantedCarMessageSeen = False Then
                    isWantedCarMessageSeen = True
                    GTA.UI.ShowSubtitle("Clever Eddie is looking for this car. Deliver it to his ~b~warehouse ~w~for a reward.", 5000)
                    DropoffLocationBlip = World.CreateBlip(DropoffLocation)
                    DropoffLocationBlip.Color = BlipColor.Blue
                    DropoffLocationBlip.ShowRoute = True
                End If
            Else
                isPlayerInWantedCar = False
                isWantedCarMessageSeen = False
                If WantedCarEntity IsNot Nothing Then
                    If WantedCarEntity.Exists Then
                        WantedCarEntity.MarkAsNoLongerNeeded()
                        WantedCarEntity = Nothing
                    End If
                End If
                If DropoffLocationBlip IsNot Nothing Then
                    If DropoffLocationBlip.Exists Then
                        DropoffLocationBlip.Remove()
                    End If
                End If
                MinigameStage = MinigameStages.LookingForCar
            End If
        Else
            If MinigameStage = MinigameStages.WaitingToExitCar Then
                isPlayerInWantedCar = False
                Exit Sub
            Else
                isPlayerInWantedCar = False
                If WantedCarEntity IsNot Nothing Then
                    If WantedCarEntity.Exists Then
                        WantedCarEntity.MarkAsNoLongerNeeded()
                        WantedCarEntity = Nothing
                    End If
                End If
                If DropoffLocationBlip IsNot Nothing Then
                    If DropoffLocationBlip.Exists Then
                        DropoffLocationBlip.Remove()
                    End If
                End If
                MinigameStage = MinigameStages.LookingForCar
            End If
        End If
    End Sub

    Public Sub checkIfPlayerIsAtDropoffLocation()
        If MinigameStage = MinigameStages.DrivingToDropoffLocation Then
            Dim distance As Single = World.GetDistance(WantedCarEntity.Position, DropoffLocation)
            If distance < 6 Then
                GTA.UI.ShowSubtitle("Exit the vehicle.", 5000)
                MinigameStage = MinigameStages.WaitingToExitCar
            End If
        End If
    End Sub

    Public Sub checkIfPlayerHasLeftWantedCar()

        If MinigameStage = MinigameStages.WaitingToExitCar Then

            Dim distance As Single = World.GetDistance(WantedCarEntity.Position, DropoffLocation)

            If Game.Player.Character.IsInVehicle = False And distance < 9 And isVehicleDelivered = False Then

                If DropoffLocationBlip IsNot Nothing Then
                    If DropoffLocationBlip.Exists Then
                        DropoffLocationBlip.Remove()
                    End If
                End If

                Dim CarHealth As Single = (WantedCarEntity.Health + 100) / 1000
                Dim EngineHealth As Single = WantedCarEntity.EngineHealth / 1000
                Dim TankHealth As Single = WantedCarEntity.PetrolTankHealth / 1000
                Dim TotalVehicleHealth As Single = CarHealth * EngineHealth * TankHealth

                Dim Reward As Integer = WantedCars(WantedCarIndex).Price * RewardPercentages(WantedCarIndex) * TotalVehicleHealth

                Dim message As String = ""
                Select Case TotalVehicleHealth * 100
                    Case Is > 98
                        message = "Beautiful! Not a single scratch! You've earned the $" & Reward.ToString("N0") & ". Thanks! -Clever Eddie"
                    Case Is > 70
                        message = "The car's at about " & Math.Round(TotalVehicleHealth * 100) & "% condition, we can buff most of that out. Your reward is $" & Reward.ToString("N0") & ". Thanks."
                    Case Is > 40
                        message = "What happened to this thing? Best I can give you for it is $" & Reward.ToString("N0") & ". -Clever Eddie"
                    Case Is > 20
                        message = "Are you kidding me? I can't resell this piece of junk! You owe me $100 because I had to have it towed away!"
                        Reward = -100
                    Case Else
                End Select

                GTA.UI.Notify(message)

                WantedCarEntity.FreezePosition = True
                WantedCarEntity.IsDriveable = False
                WantedCarEntity.EngineRunning = False
                WantedCarEntity.MarkAsNoLongerNeeded()

                Dim CurrentMoney As Integer = Game.Player.Money
                Game.Player.Money = CurrentMoney + Reward

                isVehicleDelivered = True
                WantedCars(WantedCarIndex) = Nothing
                FillWantedCarList()

                WantedCarEntity = Nothing
                isPlayerInWantedCar = False
                isWantedCarMessageSeen = False
                isVehicleDelivered = False
                WantedCarIndex = -1
                MinigameStage = MinigameStages.LookingForCar
            End If
        End If
    End Sub

    Public Sub checkIfPlayerIsDead()
        If MinigameStage = MinigameStages.DrivingToDropoffLocation Then
            If Game.Player.Character.IsDead Then
                If DropoffLocationBlip IsNot Nothing Then
                    If DropoffLocationBlip.Exists Then
                        DropoffLocationBlip.Remove()
                    End If
                End If
            End If
        End If
    End Sub

End Class

Public Class Car
    Public Name As String = ""
    Public DisplayName As String = ""
    Public Price As Integer = 0
    Public Type As CarType

    Public Enum CarType
        Motorcycle
        Compact
        Coupe
        Sedan
        Sport
        SportClassic
        Super
        Muscle
        OffRoad
        SUV
        Van
    End Enum

    Public Sub New(VehicleName As String, VehicleDisplayName As String, VehiclePrice As Integer, VehicleType As CarType)
        Name = VehicleName
        DisplayName = VehicleDisplayName
        Price = VehiclePrice
        Type = VehicleType
        CarList.Add(Me)
    End Sub
End Class

Public Module Cars

    Public CarList As New List(Of Car)

    Public AlbanyAlpha As New Car("Albany Alpha", "ALPHA", 149000, Car.CarType.Sport)
    Public AlbanyBuccaneer As New Car("Albany Buccaneer", "BUCCANEE", 11846, Car.CarType.Muscle)
    Public AlbanyEmperor As New Car("Albany Emperor", "EMPEROR", 6998, Car.CarType.Sedan)
    Public AlbanyPrimo As New Car("Albany Primo", "PRIMO", 9000, Car.CarType.Sedan)
    Public AnnisElegyRH8 As New Car("Annis Elegy RH8", "ELEGY2", 95000, Car.CarType.Sport)
    Public BenefactorDubsta As New Car("Benefactor Dubsta", "DUBSTA", 115400, Car.CarType.SUV)
    Public BenefactorFeltzer As New Car("Benefactor Feltzer", "FELTZER", 130000, Car.CarType.Sport)
    Public BenefactorPanto As New Car("Benefactor Panto", "PANTO", 13995, Car.CarType.Compact)
    Public BenefactorSchwartzer As New Car("Benefactor Schwartzer", "SCHWARTZE", 80000, Car.CarType.Sport)
    Public BravadoBanshee As New Car("Bravado Banshee", "BANSHEE", 90000, Car.CarType.Sport)
    Public BravadoGauntlet As New Car("Bravado Gauntlet", "GAUNTLET", 32000, Car.CarType.Muscle)
    Public BravadoGresley As New Car("Bravado Gresley", "GRESLEY", 29000, Car.CarType.SUV)
    Public BollokanPrairie As New Car("Bollokan Prairie", "PRAIRIE", 19000, Car.CarType.Compact)
    Public ChevalFugitive As New Car("Cheval Fugitive", "FUGITIVE", 22465, Car.CarType.Sedan)
    Public ChevalSurge As New Car("Cheval Surge", "SURGE", 38000, Car.CarType.Sedan)
    Public DeclasseAsea As New Car("Declasse Asea", "ASEA", 12000, Car.CarType.Sedan)
    Public DeclassePremier As New Car("Declasse Premier", "PREMIER", 10000, Car.CarType.Sedan)
    Public DewbaucheeExemplar As New Car("Dewbauchee Exemplar", "EXEMPLAR", 205000, Car.CarType.Coupe)
    Public DewbaucheeMassacro As New Car("Dewbauchee Massacro", "MASSACRO", 275000, Car.CarType.Sport)
    Public DinkaBlista As New Car("Dinka Blista", "BLISTA", 19500, Car.CarType.Compact)
    Public DinkaBlistaCompact As New Car("Dinka Blista Compact", "BLISTA2", 42000, Car.CarType.Sport)
    Public EnusCognoscentiCabrio As New Car("Enus Cognoscenti Cabrio", "COGCABRI", 185000, Car.CarType.Coupe)
    Public FathomFQ2 As New Car("Fathom FQ 2", "FQ2", 45850, Car.CarType.SUV)
    Public GallivanterBaller As New Car("Gallivanter Baller", "BALLER", 50000, Car.CarType.SUV)
    Public GallivanterBaller2 As New Car("Gallivanter Baller (2nd Gen)", "BALLER2", 90000, Car.CarType.SUV)
    Public GrottiCarbonizzare As New Car("Grotti Carbonizzare", "CARBONIZ", 195000, Car.CarType.Sport)
    Public GrottiCheetah As New Car("Grotti Cheetah", "CHEETAH", 650000, Car.CarType.Super)
    Public GrottiTurismoR As New Car("Grotti Turismo R", "TURISMOR", 500000, Car.CarType.Super)
    Public HijakKhamelion As New Car("Hijak Khamelion", "KHAMEL", 100000, Car.CarType.Sport)
    Public InveteroCoquette As New Car("Invetero Coquette", "COQUETTE", 138000, Car.CarType.SportClassic)
    Public InveteroCoquetteClassic As New Car("Invetero Coquette Classic", "COQUETTE2", 665000, Car.CarType.SportClassic)
    Public KarinAsterope As New Car("Karin Asterope", "ASTROPE", 26000, Car.CarType.Sedan)
    Public KarinBJXL As New Car("Karin BeeJay XL", "BJXL", 27680, Car.CarType.SUV)
    Public KarinDilettante As New Car("Karin Dilettante", "DILETTANTE", 25000, Car.CarType.Compact)
    Public KarinFuto As New Car("Karin Futo", "FUTO", 3687, Car.CarType.Sport)
    Public KarinIntruder As New Car("Karin Intruder", "INTRUDER", 15900, Car.CarType.Sedan)
    Public KarinKuruma As New Car("Karin Kuruma", "KURUMA", 34995, Car.CarType.Sport)
    Public KarinSultan As New Car("Karin Sultan", "SULTAN", 26295, Car.CarType.Sport)
    Public LampadatiCasco As New Car("Lampadati Casco", "CASCO", 680000, Car.CarType.SportClassic)
    Public LampadatiFelon As New Car("Lampadati Felon", "FELON", 90000, Car.CarType.Coupe)
    Public LampadatiFelonGT As New Car("Lampadati Felon GT", "FELON2", 95000, Car.CarType.Coupe)
    Public LampadatiFuroreGT As New Car("Lampadati Furore GT", "FURORE", 448000, Car.CarType.Sport)
    Public MaibatsuPenumbra As New Car("Maibatsu Penumbra", "PENUMBRA", 24000, Car.CarType.Sport)
    Public Obey9F As New Car("Obey 9F", "NINEF", 120000, Car.CarType.Sport)
    Public Obey9F2 As New Car("Obey 9F Cabrio", "NINEF2", 130000, Car.CarType.Sport)
    Public OcelotF620 As New Car("Ocelot F620", "F620", 80000, Car.CarType.Coupe)
    Public OcelotJackal As New Car("Ocelot Jackal", "JACKAL", 60000, Car.CarType.Coupe)
    Public ÖverflödEntityXF As New Car("Overflod Entity XF", "ENTITYXF", 795000, Car.CarType.Super)
    Public PegassiInfernus As New Car("Pegassi Infernus", "INFERNUS", 439000, Car.CarType.Super)
    Public PegassiMonroe As New Car("Pegassi Monroe", "MONROE", 489000, Car.CarType.SportClassic)
    Public PegassiVacca As New Car("Pegassi Vacca", "VACCA", 239000, Car.CarType.Super)
    Public PegassiZentorno As New Car("Pegassi Zentorno", "ZENTORNO", 725000, Car.CarType.Super)
    Public PfisterComet As New Car("Pfister Comet", "COMET2", 85000, Car.CarType.Sport)
    Public SchysterFusilade As New Car("Schyster Fusilade", "FUSILADE", 36000, Car.CarType.Sport)
    Public TruffadeAdder As New Car("Truffade Adder", "ADDER", 599000, Car.CarType.Super)
    Public UbermachtZion As New Car("Ubermacht Zion", "ZION", 50000, Car.CarType.Coupe)
    Public UbermachtOracle As New Car("Ubermacht Oracle", "ORACLE", 80000, Car.CarType.Sedan)
    Public VapidBullet As New Car("Vapid Bullet", "BULLET", 155000, Car.CarType.Super)
    Public VapidDominator As New Car("Vapid Dominator", "DOMINATO", 35000, Car.CarType.Muscle)
    Public VapidStanier As New Car("Vapid Stanier", "STANIER", 10000, Car.CarType.Sedan)
    Public WeenyIssi As New Car("Weeny Issi", "ISSI2", 18000, Car.CarType.Compact)
    Public ZirconiumStratum As New Car("Zirconium Stratum", "STRATUM", 10000, Car.CarType.Sedan)

End Module
